--!parent
--!encoding=utf-8

--****************************************************************************
-- @file			LSW CORE LIB: SPRITE
-- @author   	Valentin Schmidt
-- @version  	0.4
-- @desc
-- Simple sprite abstraction, factory method returns new sprites with a classic 
-- EventEmitter interface (addListener, removeListener, ...)
--****************************************************************************

--global $

property _DispatcherClass
property _nextChannel

----------------------------------------
-- @constructor
----------------------------------------
on new (me)
	me._DispatcherClass = me.$.include(me.libdir&"dispatcher.ls")
	me._nextChannel = 1
	return me
end

----------------------------------------
-- FACTORY METHOD: creates and returns a new "scripted" sprite
----------------------------------------
on make (me, mem, props, dispatchEvents)
	
--	if voidP(loc) then s = channel(me._nextChannel).makeScriptedSprite(mem)
--	else s = channel(me._nextChannel).makeScriptedSprite(mem, loc)
	
	s = channel(me._nextChannel).makeScriptedSprite(mem)
	if ilk(props)=#propList then
		cnt = count(props)
		repeat with i = 1 to cnt
			do("s."&props.getPropAt(i)&"=props[i]")
		end repeat
	end if
	
	if dispatchEvents then
		obj = me._DispatcherClass.new()
		obj.spriteNum = me._nextChannel
		s.scriptInstanceList.add(obj)
	end if
	me._nextChannel = me._nextChannel + 1
	return s
end

----------------------------------------
-- 
----------------------------------------
on delete (me, spriteNumOrRef)
	if ilk(spriteNumOrRef)=#sprite then spriteNumOrRef = spriteNumOrRef.spriteNum
	sprite(spriteNumOrRef).member = member(0)
	channel(spriteNumOrRef).removeScriptedSprite()
end

----------------------------------------
-- create member, set some defaults that make sense
----------------------------------------
on newMember (me, type, props)
	m = new(type)
	case (type) of
		#button:
			m.text = "OK"
			m.alignment = "center"
		#field:
			m.rect = rect(0,0,100,20)
			m.border = 1
			m.editable = 1
		#shape:
			m.rect = rect(0,0,100,100)
			m.shapeType = #rect
			m.filled = 1
			m.pattern = 1

	end case
	if ilk(props)=#propList then
		cnt = count(props)
		repeat with i = 1 to cnt
			do("m."&props.getPropAt(i)&"=props[i]")
		end repeat
	end if
	return m
end
